///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: $
// Version        : $Revision: $
// Function       : 
//
// Author         : $Author: $
// Date           : $Date: $
//
// Notes          : Asynchronous COM wrapper for Win32 Mailslots.
//
// Modifications  :
//
// $Log: $
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "JBCOMMailslot.h"
#include "AsyncCOMMailslot.h"

#include "Utils.h"

#include <process.h>

///////////////////////////////////////////////////////////////////////////////
// CAsyncCOMMailslot
///////////////////////////////////////////////////////////////////////////////

CAsyncCOMMailslot::CAsyncCOMMailslot(
   const GUID &clsid,
   const IID &iid)
   :  CCOMMailslot(clsid, iid),
      m_hThread(INVALID_HANDLE_VALUE),
      m_pIStream(0),
      m_dwNumberOfBytesRead(0),
      m_pIEventHelper(0)
{
}
   
CAsyncCOMMailslot::~CAsyncCOMMailslot()
{
   if (m_hThread != INVALID_HANDLE_VALUE)
   {
      m_shutdown.Set();

      WaitWithMessageLoop(m_hThread);

      ::CloseHandle(m_hThread);
   }

   if (m_pIEventHelper)
   {
      m_pIEventHelper->Release();
   }
}

HRESULT CAsyncCOMMailslot::Create(
   LPOLESTR name,
   DWORD dwMaxMessageSize,
   IUnknown *pUnkSink)
{
   HRESULT hr = CCOMMailslot::Create(name, dwMaxMessageSize, MAILSLOT_WAIT_FOREVER);

   if (SUCCEEDED(hr))
   {
      hr = EnableAsyncReceives(pUnkSink);
   }

   return hr;
}

// Spawns a thread to deal with the async reads on the mailslot and marshals the event interface
// across to the thread so that it can raise events on the main thread.

HRESULT CAsyncCOMMailslot::EnableAsyncReceives(
   IUnknown *pUnkSink)
{
   HRESULT hr = S_OK;

   if (!m_shutdown.Create())
   {
      hr = E_FAIL;
   }

   _AsyncServerEvent *pIEventHelper = 0;

   if (SUCCEEDED(hr))
   {
      hr = pUnkSink->QueryInterface(IID__AsyncServerEvent, (void**)&pIEventHelper);
   }

   if (SUCCEEDED(hr))
   {
      hr = ::CoMarshalInterThreadInterfaceInStream(IID__AsyncServerEvent, pIEventHelper, &m_pIStream);
      
      pIEventHelper->Release();
   }

   if (SUCCEEDED(hr))
   {
      unsigned int threadID = 0;

      m_hThread = (HANDLE)::_beginthreadex(0, 0, ThreadFunction, this, 0, &threadID);

      if (0 == m_hThread)
      {
         hr = E_FAIL;

         m_hThread = INVALID_HANDLE_VALUE;
      }
   }

   return hr;
}

unsigned int __stdcall CAsyncCOMMailslot::ThreadFunction(void *pV)
{
   unsigned int result = 0;

   if (pV)
   {
      ::CoInitialize(0);

      CAsyncCOMMailslot &me = *(CAsyncCOMMailslot*)pV;

      result = me.RunThread();
         
      ::CoUninitialize();
   }

   return result;
}

unsigned int CAsyncCOMMailslot::RunThread()
{
   m_pIEventHelper = 0;

   HRESULT hr = ::CoGetInterfaceAndReleaseStream(m_pIStream, IID__AsyncServerEvent, (void**)&m_pIEventHelper );

   if (FAILED(hr))
   {
      return 0;
   }

   SetMailslotInfo(m_hMailslot, MAILSLOT_WAIT_FOREVER);

   if (!m_overlapped.CreateEvent())
   {
      return 0;
   }

   HANDLE HandlesToWaitFor[2];
   HandlesToWaitFor[0] = m_shutdown.GetHANDLE();
   HandlesToWaitFor[1] = m_overlapped.hEvent;

   if (SetupRead())
   {
      bool ok = true;
   
      do
      {
         DWORD waitResult = ::WaitForMultipleObjects(2, HandlesToWaitFor, FALSE, INFINITE);

         switch (waitResult)
         {
            case WAIT_OBJECT_0:     // Signal to end the thread.
            
               ok = false;
            
            break;

            case WAIT_OBJECT_0 + 1: // Read 

               ok = HandleRead();

               if (ok)
               {
                  ok = SetupRead();
               }
            
            break;

         }
      }
      while (ok);
   }
   
   ::CancelIo(m_hMailslot);

   m_pIEventHelper->Release();

   m_pIEventHelper = 0;

   return 0;
}

bool CAsyncCOMMailslot::SetupRead()
{
   bool ok = true;

   bool dataNotPending;

   do
   {
      if (WAIT_TIMEOUT != WaitForSingleObject(m_shutdown.GetHANDLE(),0))
      {
         return false;
      }
   
      if (m_maxMessageSize == 0)
      {
         // If the maximum message size wasnt set when the mailslot was created then we
         // perform a read with a 0 byte buffer, once a message is available the read
         // will complete and we will handle the corresponding ERROR_INSUFFICIENT_BUFFER error
         // from GetOverlappedResult() in HandleRead().

         dataNotPending = (0 != ::ReadFile(m_hMailslot, 0, 0, 0, &m_overlapped));
      }
      else
      {
         // We know the maximum message size, the buffer is already allocated, we can read
         // directly into the buffer...

         BYTE *pData;
         DWORD dwDataLength = 0;

         GetBuffer(&pData, &dwDataLength);
      
         dataNotPending = (0 != ::ReadFile(m_hMailslot, pData, dwDataLength, &dwDataLength, &m_overlapped));
      }

      if (dataNotPending)
      {
         // This could only happen if there was data waiting to be read. Process it and loop around again...

         ok = HandleRead();
      }
   }
   while (dataNotPending && ok);

   if (ok)
   {
      // ReadFile failed.  Expected because of overlapped I/O.
      
      DWORD dwLastError = GetLastError();

      if (dwLastError != ERROR_IO_PENDING)
      {
         ok = false;

         OutputDebugString(_T("CAsyncCOMMailslot::SetupRead - Unexpected error\n"));
         OutputDebugString(GetLastErrorMessage(dwLastError));
         OutputDebugString(_T("\n"));
      }
   }

   return ok;
}

bool CAsyncCOMMailslot::HandleRead()
{
   bool ok = true;

   m_dwNumberOfBytesRead = 0;

   if (::GetOverlappedResult(m_hMailslot, &m_overlapped, &m_dwNumberOfBytesRead, false))
   {
      // A read has completed ...

      m_pIEventHelper->OnDataRecieved();
   }
   else
   {
      DWORD lastError = ::GetLastError();

      if (lastError == ERROR_INSUFFICIENT_BUFFER)
      {
         // We have a mailslot without a maximum message size, we posted a read for
         // 0 bytes and there's now a message available...

         BYTE *pData;

         HRESULT hr = CCOMMailslot::Read(&pData, m_dwNumberOfBytesRead);

         if (SUCCEEDED(hr))
         {
            m_pIEventHelper->OnDataRecieved();
         }
         else
         {
            OutputDebugString(_T("CAsyncCOMMailslot::HandleRead - Unexpected error\n"));
            OutputDebugString(GetLastErrorMessage(hr));
            OutputDebugString(_T("\n"));

            ok = false;
         }
      }
      else
      {
         OutputDebugString(_T("CAsyncCOMMailslot::HandleRead - Unexpected error\n"));
         OutputDebugString(GetLastErrorMessage(lastError));
         OutputDebugString(_T("\n"));

         ok = false;
      }
   }

   return ok;
}

BYTE *CAsyncCOMMailslot::GetData()
{
   return m_readBuffer.GetBuffer();
}

DWORD CAsyncCOMMailslot::GetDataLength()
{
   return m_dwNumberOfBytesRead;
}

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
