///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: $
// Version        : $Revision: $
// Function       : 
//
// Author         : $Author: $
// Date           : $Date: $
//
// Notes          : Asynchronous server mailslot COM object.
//
// Modifications  :
//
// $Log: $
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "JBCOMMailslot.h"
#include "AsyncServerMailslot.h"

#include "MailslotData.h"

#include "Utils.h"

///////////////////////////////////////////////////////////////////////////////
// CAsyncServerMailslot
///////////////////////////////////////////////////////////////////////////////

CAsyncServerMailslot::CAsyncServerMailslot()
   :  CAsyncCOMMailslot(CLSID_AsyncServerMailslot, IID_IAsyncServerMailslot),
      m_asyncServerEventHelper(*this),
      m_pIData(0),
      m_pIDataInit(0)
{
}
   
CAsyncServerMailslot::~CAsyncServerMailslot()
{
   if (m_pIDataInit)
   {
      m_pIDataInit->Release();
   }

   if (m_pIData)
   {
      m_pIData->Release();
   }
}

STDMETHODIMP CAsyncServerMailslot::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IAsyncServerMailslot
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CAsyncServerMailslot::get_MaxMessageSize(long *pVal)
{
   return MaxMessageSize(pVal);
}

STDMETHODIMP CAsyncServerMailslot::Init(
   /*[in]*/ BSTR name,
   /*[in]*/ DWORD dwMaxMessageSize,
   /*[in]*/ DWORD dwReadTimeout)
{
   if (dwReadTimeout != MAILSLOT_WAIT_FOREVER)
   {
      return E_INVALIDARG;
   }

   HRESULT hr = Create(name, dwMaxMessageSize, &m_asyncServerEventHelper);

   if (SUCCEEDED(hr))
   {
      hr = CMailslotData::CreateInstance(&m_pIDataInit);

      if (SUCCEEDED(hr))
      {
         hr = m_pIDataInit->QueryInterface(&m_pIData);
      }
   }

   return hr;
}

STDMETHODIMP CAsyncServerMailslot::OnDataRecieved()
{
   if (!m_pIDataInit || !m_pIData)
   {
      return ReportError(_T("Internal error: failed to create MailslotData object"), E_UNEXPECTED);
   }
    
   HRESULT hr = m_pIDataInit->Init(GetData(), GetDataLength());

   if (SUCCEEDED(hr))
   {  
      Fire_OnDataRecieved(m_pIData);   /// this can stall if the handler doesnt return - 
                                       // the ATL implementation is difficult to multi thread...
   }

   return hr;
}

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
