///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: $
// Version        : $Revision: $
// Function       : 
//
// Author         : $Author: $
// Date           : $Date: $
//
// Notes          : COM wrapper for Win32 Mailslots. We could break out the
//                  server specific stuff so that the client objects dont 
//                  include the excess baggage, but it hardly seems worth it.
//
// Modifications  :
//
// $Log: $
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "COMMailslot.h"

#include "Utils.h"

///////////////////////////////////////////////////////////////////////////////
// Constants
///////////////////////////////////////////////////////////////////////////////

const size_t MIN_BUFFER_SIZE = 400;

///////////////////////////////////////////////////////////////////////////////
// CCOMMailslot
///////////////////////////////////////////////////////////////////////////////

CCOMMailslot::CCOMMailslot(
   const GUID &clsid,
   const IID &iid)
   :  m_hMailslot(INVALID_HANDLE_VALUE),
      m_clsid(clsid),
      m_iid(iid),
      m_maxMessageSize(0)
{
}
   
CCOMMailslot::~CCOMMailslot()
{
   if (IsOpen())
   {
      ::CloseHandle(m_hMailslot);
      m_hMailslot = INVALID_HANDLE_VALUE;
   }
}

HRESULT CCOMMailslot::Open(
   LPOLESTR pName,
   LPOLESTR pComputerOrDomain)
{
   HRESULT hr = CheckMailslotIsNotOpen();

   if (SUCCEEDED(hr))
   {
      if (wcslen(pComputerOrDomain) + wcslen(pName) > _MAX_PATH - 13)
      {
         hr = ReportError(L"Total length of mailslot name and computer/domain name exceeds maximum allowed", E_UNEXPECTED);
      }
   }

   if (SUCCEEDED(hr))
   {
      TCHAR mailslotName[_MAX_PATH];

      wsprintf(mailslotName, _T("\\\\%ls\\mailslot\\%ls"), pComputerOrDomain, pName);

      m_hMailslot = CreateFile(
         mailslotName,                          // filename 
         GENERIC_WRITE,                         // access
         FILE_SHARE_READ | FILE_SHARE_WRITE,    // share mode
         0,                                     // security
         OPEN_EXISTING,                         // creation
         FILE_ATTRIBUTE_NORMAL,                 // attributes
         0);                                    // template

      if (m_hMailslot == INVALID_HANDLE_VALUE)
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}

HRESULT CCOMMailslot::Create(
   LPOLESTR pName,
   DWORD dwMaxMessageSize,
   DWORD dwReadTimeout)
{
   HRESULT hr = CheckMailslotIsNotOpen();

   if (SUCCEEDED(hr))
   {
      if (wcslen(pName) > _MAX_PATH - 14)
      {
         hr = ReportError(L"Total length of mailslot name exceeds maximum allowed", E_UNEXPECTED);
      }
   }

   if (SUCCEEDED(hr))
   { 
      TCHAR mailslotName[_MAX_PATH];

      wsprintf(mailslotName, _T("\\\\.\\mailslot\\%ls"), pName);

      m_hMailslot = CreateMailslot(
         mailslotName, 
         dwMaxMessageSize, 
         dwReadTimeout,                
         0);                           // security

      if (m_hMailslot == INVALID_HANDLE_VALUE)
      {
         hr = ReportError(::GetLastError());   
      }
   }

   if (SUCCEEDED(hr))
   {
      m_maxMessageSize = dwMaxMessageSize;
   }

   return hr;
}

HRESULT CCOMMailslot::CheckMailslotIsNotOpen()
{
   HRESULT hr = S_OK;

   if (IsOpen())
   {
      hr = ReportError(L"Internal error: Mailslot is already open", E_UNEXPECTED);
   }

   return hr;
}

HRESULT CCOMMailslot::CheckMailslotIsOpen()
{
   HRESULT hr = S_OK;

   if (!IsOpen())
   {
      hr = ReportError(L"Internal error: Mailslot is not open", E_UNEXPECTED);
   }

   return hr;
}

bool CCOMMailslot::IsOpen() const
{
  return (m_hMailslot != INVALID_HANDLE_VALUE);
}

HRESULT CCOMMailslot::Write(
   LPCVOID pBuffer,
   DWORD dwNumberOfBytesToWrite)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      DWORD dwNumberOfBytesWritten = 0;

      BOOL bSuccess = WriteFile(
         m_hMailslot, 
         pBuffer, 
         dwNumberOfBytesToWrite,
         &dwNumberOfBytesWritten, 
         0);                           // OVERLAPPED

      if (bSuccess)
      {
         if (dwNumberOfBytesWritten != dwNumberOfBytesToWrite)
         {
            hr = ReportError(L"Failed to write all of the data", E_UNEXPECTED);
         }
      }
      else
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}

HRESULT CCOMMailslot::Read(
   BYTE **ppData, 
   DWORD &dwDataLength)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      if (!ppData)
      {
         hr = ReportError(L"pData is an invalid pointer", E_POINTER);
      }
   }

   DWORD dwBufferSize = 0;

   if (SUCCEEDED(hr))
   {
      hr = GetBuffer(ppData, &dwBufferSize);
   }

   if (SUCCEEDED(hr))
   {
      if (!::ReadFile(m_hMailslot, *ppData, dwBufferSize, &dwDataLength, 0))
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}

HRESULT CCOMMailslot::SizeOfWaitingMessage(long *pVal)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      if (!pVal)
      {
         hr = ReportError(L"pVal is an invalid pointer", E_POINTER);
      }
   }

   if (SUCCEEDED(hr))
   {
      *pVal = 0;

      DWORD dwSize = 0;
   
      if (::GetMailslotInfo(m_hMailslot, NULL, &dwSize, NULL, NULL))
      {
         if (dwSize != MAILSLOT_NO_MESSAGE)
         {
            *pVal = dwSize;
         }
         else
         {
            hr = ReportError(L"There is no message waiting", E_UNEXPECTED);
         }
      }
      else
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}

HRESULT CCOMMailslot::MessageWaiting(BOOL *pVal)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      if (!pVal)
      {
         hr = ReportError(L"pVal is an invalid pointer", E_POINTER);
      }
   }

   if (SUCCEEDED(hr))
   {
      long msgsWaiting = 0;

      hr = MessageCount(&msgsWaiting);

      if (SUCCEEDED(hr))
      {
         *pVal = (msgsWaiting != 0);
      }
   }

   return hr;
}

HRESULT CCOMMailslot::MessageCount(long *pVal)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      if (!pVal)
      {
         hr = ReportError(L"pVal is an invalid pointer", E_POINTER);
      }
   }

   if (SUCCEEDED(hr))
   {
      *pVal = 0;

      DWORD dwCount = 0;
   
      if (::GetMailslotInfo(m_hMailslot, NULL, NULL, &dwCount, NULL))
      {
         *pVal = (dwCount != 0);
      }
      else
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}

HRESULT CCOMMailslot::MaxMessageSize(long *pVal)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      if (!pVal)
      {
         hr = ReportError(L"pVal is an invalid pointer", E_POINTER);
      }
   }

   if (SUCCEEDED(hr))
   {
      *pVal = 0;

      DWORD dwMaxSize = 0;
   
      if (::GetMailslotInfo(m_hMailslot, &dwMaxSize, NULL, NULL, NULL))
      {
         *pVal = dwMaxSize;
      }
      else
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}

HRESULT CCOMMailslot::ReadTimeout(long *pVal)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      if (!pVal)
      {
         hr = ReportError(L"pVal is an invalid pointer", E_POINTER);
      }
   }

   if (SUCCEEDED(hr))
   {
      *pVal = 0;

      DWORD dwReadTimeout = 0;

      if (::GetMailslotInfo(m_hMailslot, NULL, NULL, NULL, &dwReadTimeout))
      {
         *pVal = dwReadTimeout;
      }
      else
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}

HRESULT CCOMMailslot::ReadTimeout(long newVal)
{
   HRESULT hr = CheckMailslotIsOpen();

   if (SUCCEEDED(hr))
   {
      if (!::SetMailslotInfo(m_hMailslot, newVal))
      {
         hr = ReportError(::GetLastError());   
      }
   }

   return hr;
}


HRESULT CCOMMailslot::ReportError(DWORD lastError)
{
   return AtlReportError(m_clsid, GetLastErrorMessage(lastError), m_iid, DISP_E_EXCEPTION);
}

HRESULT CCOMMailslot::ReportError(
   LPCTSTR pMessage,
   HRESULT hr)
{
   return AtlReportError(m_clsid, pMessage, m_iid, hr);
}

HRESULT CCOMMailslot::ReportError(
   LPOLESTR pMessage,
   HRESULT hr)
{
   return AtlReportError(m_clsid, pMessage, m_iid, hr);
}

HRESULT CCOMMailslot::GetBuffer(
   BYTE **ppBuffer,
   DWORD *pdwBufferSize)
{
   if (!ppBuffer)
   {
      return ReportError(L"ppBuffer is an invalid pointer", E_POINTER);
   }

   if (!pdwBufferSize)
   {
      return ReportError(L"pdwBufferSize is an invalid pointer", E_POINTER);
   }

   *ppBuffer = 0;
   *pdwBufferSize = 0;

   long requiredSize = m_maxMessageSize;

   HRESULT hr = S_OK;

   if (requiredSize == 0)
   {
      // mailslot wasnt created with a max message size so we have to be able to handle 
      // any size of message...

      hr = SizeOfWaitingMessage(&requiredSize);

      if (requiredSize < MIN_BUFFER_SIZE)
      {
         requiredSize = MIN_BUFFER_SIZE;
      }
   }

   if (SUCCEEDED(hr))
   {
      if (!m_readBuffer.Resize(requiredSize))
      {
         hr = ReportError(L"Unable to allocate enough memory to read mailslot data", E_OUTOFMEMORY);
      }
   }

   if (SUCCEEDED(hr))
   {
      *ppBuffer = m_readBuffer.GetBuffer();
      *pdwBufferSize = m_readBuffer.GetBufferSize();
   }

   return hr;
}

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
