///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: $
// Version        : $Revision: $
// Function       : 
//
// Author         : $Author: $
// Date           : $Date: $
//
// Notes          : Client Mailslot COM object.
//
// Modifications  :
//
// $Log: $
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "JBCOMMailslot.h"
#include "ClientMailslot.h"

#include "Utils.h"

#include <comdef.h>

///////////////////////////////////////////////////////////////////////////////
// CClientMailslot
///////////////////////////////////////////////////////////////////////////////

CClientMailslot::CClientMailslot()
   : CCOMMailslot(CLSID_ClientMailslot, IID_IClientMailslot)
{
}

STDMETHODIMP CClientMailslot::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IClientMailslot
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CClientMailslot::WriteString(
   /*[in]*/ BSTR data)
{
   return CCOMMailslot::Write(data, ::SysStringByteLen(data));
}

STDMETHODIMP CClientMailslot::Write(
   /*[in]*/ VARIANT arrayOfBytes)
{
   if (arrayOfBytes.vt != (VT_ARRAY | VT_UI1)) 
   {
      return ReportError(L"Expected an array of bytes", E_INVALIDARG);
   }

   const long dims = ::SafeArrayGetDim(arrayOfBytes.parray);

   if (dims != 1) 
   {
      return ReportError(L"Expected a one dimensional array", E_INVALIDARG);
   }

   long upperBounds;
   long lowerBounds;

   ::SafeArrayGetLBound(arrayOfBytes.parray, 1, &lowerBounds);
   ::SafeArrayGetUBound(arrayOfBytes.parray, 1, &upperBounds);

   const size_t size = upperBounds - lowerBounds;

   BYTE *pBuff;

   ::SafeArrayAccessData(arrayOfBytes.parray, (void**)&pBuff);

   HRESULT hr = CCOMMailslot::Write(pBuff, size);

   ::SafeArrayUnaccessData(arrayOfBytes.parray);

   return hr;
}

STDMETHODIMP CClientMailslot::Init(
   /*[in]*/ BSTR name,
   /*[in]*/ VARIANT computerOrDomain)
{
   LPOLESTR lpCompOrDomain = L".";

   if (computerOrDomain.vt == VT_BSTR)
   {
      lpCompOrDomain = computerOrDomain.bstrVal;
   }

   return Open(name, lpCompOrDomain);
}

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
