///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: $
// Version        : $Revision: $
// Function       : 
//
// Author         : $Author: $
// Date           : $Date: $
//
// Notes          : COM Mailslot object factory.
//
// Modifications  :
//
// $Log: $
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "JBCOMMailslot.h"
#include "MailslotFactory.h"

#include "ClientMailslot.h"
#include "ServerMailslot.h"
#include "AsyncServerMailslot.h"

#include "Utils.h"

///////////////////////////////////////////////////////////////////////////////
// CMailslotFactory
///////////////////////////////////////////////////////////////////////////////

STDMETHODIMP CMailslotFactory::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IMailslotFactory
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CMailslotFactory::CreateClientMailslot(
   BSTR name,
   VARIANT computerOrDomain,
   IClientMailslot **ppSlot)
{
   if (ppSlot == 0)
   {
      return AtlReportError(CLSID_MailslotFactory, L"ppSlot is an invalid pointer", IID_IMailslotFactory, E_POINTER);
   }

   *ppSlot = 0;

   if (computerOrDomain.vt != VT_BSTR && 
       computerOrDomain.vt != VT_EMPTY &&
       computerOrDomain.vt != VT_ERROR)
   {
      return E_INVALIDARG;
   }
   
   IClientMailslotInit *pInit = 0;

   HRESULT hr = CClientMailslot::CreateInstance(&pInit);

   if (SUCCEEDED(hr))
   {
      hr = pInit->Init(name, computerOrDomain);

      if (SUCCEEDED(hr))
      {  
         hr = pInit->QueryInterface(ppSlot);
      }
      pInit->Release();
   }

   return hr;
}

STDMETHODIMP CMailslotFactory::CreateServerMailslot(
   BSTR name,
   VARIANT maxMessageSize,
   VARIANT readTimeOut,
   IServerMailslot **ppSlot)
{
   HRESULT hr = S_OK;

   if (ppSlot == 0)
   {
      return AtlReportError(CLSID_MailslotFactory, L"ppSlot is an invalid pointer", IID_IMailslotFactory, E_POINTER);
   }

   *ppSlot = 0;

   DWORD dwMaxMessageSize = 0;
   
   hr = GetOptionalDWORD(maxMessageSize, dwMaxMessageSize);

   if (FAILED(hr))
   {
      return hr;
   }
   
   DWORD dwReadTimeOut = 0;
      
   hr = GetOptionalDWORD(readTimeOut, dwReadTimeOut);

   if (FAILED(hr))
   {
      return hr;
   }

   IServerMailslotInit *pInit = 0;

   hr = CServerMailslot::CreateInstance(&pInit);

   if (SUCCEEDED(hr))
   {
      hr = pInit->Init(name, dwMaxMessageSize, dwReadTimeOut);

      if (SUCCEEDED(hr))
      {
         hr = pInit->QueryInterface(ppSlot);
      }

      pInit->Release();
   }

   return hr;
}

STDMETHODIMP CMailslotFactory::CreateAsyncServerMailslot(
   BSTR name,
   VARIANT maxMessageSize,
   IAsyncServerMailslot **ppSlot)
{
   HRESULT hr = S_OK;

   if (ppSlot == 0)
   {
      return AtlReportError(CLSID_MailslotFactory, L"ppSlot is an invalid pointer", IID_IMailslotFactory, E_POINTER);
   }

   *ppSlot = 0;

   DWORD dwMaxMessageSize = 0;
   
   hr = GetOptionalDWORD(maxMessageSize, dwMaxMessageSize);

   if (FAILED(hr))
   {
      return hr;
   }
   
   IServerMailslotInit *pInit = 0;

   hr = CAsyncServerMailslot::CreateInstance(&pInit);

   if (SUCCEEDED(hr))
   {
      hr = pInit->Init(name, dwMaxMessageSize, MAILSLOT_WAIT_FOREVER);

      if (SUCCEEDED(hr))
      {
         hr = pInit->QueryInterface(ppSlot);
      }

      pInit->Release();
   }

   return hr;
}

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
