///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: $
// Version        : $Revision: $
// Function       : 
//
// Author         : $Author: $
// Date           : $Date: $
//
// Notes          : Misc bits and pieces...
//
// Modifications  :
//
// $Log: $
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "utils.h"

const LPTSTR GetLastErrorMessage(DWORD last_error)
{
   static TCHAR errmsg[512];

   if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 
      0,
      last_error,
      0,
      errmsg, 
      sizeof(errmsg)-1,
      NULL))
   {
      /* if we fail, call ourself to find out why and return that error */
      return (GetLastErrorMessage(GetLastError()));  
   }

   return(errmsg);
}

HRESULT GetOptionalDWORD(
   VARIANT &source, 
   DWORD &result, 
   const DWORD defaultValue /* = 0 */)
{
   result = defaultValue;

   if (source.vt != VT_EMPTY && source.vt != VT_ERROR)
   {
      VARIANT dest;
         
      ::VariantInit(&dest);

      HRESULT hr = ::VariantChangeType(&dest, &source, 0, VT_I4);

      if (FAILED(hr))
      {
         return hr;
      }
      
      result = dest.lVal;

      ::VariantClear(&dest);
   }

   return S_OK;
}

bool WaitWithMessageLoop(HANDLE hEvent)
{
   MSG msg;
   
   while(1)       
   {
      DWORD dwRet = MsgWaitForMultipleObjects( 1,    // One event to wait for
         &hEvent,        // The array of events
         FALSE,          // Wait for 1 event
         INFINITE,       // Timeout value
         QS_ALLINPUT);   // Any message wakes up
       
      if(dwRet == WAIT_OBJECT_0)       
      {
         // The event was signaled, return         
         return true;
      } 
      else if(dwRet == WAIT_OBJECT_0 + 1)       
      {
         // There is a window message available. Dispatch it.
         while(PeekMessage(&msg,NULL,NULL,NULL,PM_REMOVE))       
         {
            TranslateMessage(&msg);          
            DispatchMessage(&msg);
         }       
      } 
      else       
      {          // Something else happened. Return.
         return false;       
      }       
   }   
} 

HRESULT CreateSafeArray(
   BYTE *pData,
   DWORD dataLength,
   VARIANT *ppResults)
{
   SAFEARRAYBOUND sab;
   sab.cElements = dataLength;
   sab.lLbound = 0;

   SAFEARRAY *pSa = SafeArrayCreateEx(VT_UI1, 1, &sab, 0);
   
   PVOID pvData;
   
   HRESULT hr = SafeArrayAccessData(pSa, &pvData);
   
   if (SUCCEEDED(hr))
   {
      ::CopyMemory(pvData, pData, dataLength);
   
      hr = ::SafeArrayUnaccessData(pSa);
   }

   if (SUCCEEDED(hr))
   {
      ::VariantInit(ppResults);
           
      (*ppResults).parray = pSa;
      (*ppResults).vt = VT_ARRAY | VT_UI1;
   }

   return hr;
}

// these helpers have two stage construction because we dont want to use
// C++ exceptions...

COverlapped::COverlapped()
{
   ::ZeroMemory(this, sizeof(OVERLAPPED));
   hEvent = INVALID_HANDLE_VALUE;
}

COverlapped::~COverlapped()
{
   if (hEvent != INVALID_HANDLE_VALUE)
   {
      ::CloseHandle(hEvent);
   }
}

bool COverlapped::CreateEvent()
{
   if (hEvent == INVALID_HANDLE_VALUE)
   {
      hEvent = ::CreateEvent(0, true, true, 0);

      if (hEvent != 0)
      {
         return true;
      }
   }

   return false;
}

CManualResetEvent::CManualResetEvent()
   :  m_hEvent(INVALID_HANDLE_VALUE)
{
   
}

CManualResetEvent::~CManualResetEvent()
{
   if (m_hEvent != INVALID_HANDLE_VALUE)
   {
      ::CloseHandle(m_hEvent);
   }
}

bool CManualResetEvent::Create()
{
   m_hEvent = ::CreateEvent(0, TRUE, FALSE, 0);

   return (0 != m_hEvent);
}

bool CManualResetEvent::Set()
{
   return (0 != ::SetEvent(m_hEvent));
}

HANDLE CManualResetEvent::GetHANDLE() const
{
   return m_hEvent;
}

CBuffer::CBuffer()
   :  m_pBuffer(0),
      m_size(0)
{
}

CBuffer::~CBuffer()
{
   HeapFree(GetProcessHeap(), 0, m_pBuffer);
}

bool CBuffer::Resize(
   size_t size)
{
   bool ok = true;

   if (m_size < size)
   {
      if (size < 2 * m_size)
      {
         size = 2 * m_size;
      }

      HeapFree(GetProcessHeap(), 0, m_pBuffer);

      m_pBuffer = (BYTE*)HeapAlloc(GetProcessHeap(), 0, size);

      if (!m_pBuffer)
      {
         m_size = 0;
         ok = false;
      }
      else
      {
         m_size = size;
      }
   }

   return ok;
}


BYTE *CBuffer::GetBuffer() const
{
   return m_pBuffer;
}

size_t CBuffer::GetBufferSize() const
{
   return m_size;
}

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
